/*Usage: 
exec SQLZealot_WhoAmI 'authors' 
*/ 
CREATE PROC SQLZealot_WhoAmI 
( @TableName varchar ( 500 ) ) 
AS 
BEGIN 

    --Table Details 
    SELECT 
    @TableName 'TableName' , 
    A.Object_Id , 
    CASE 
        WHEN A.index_id = 0 THEN 'Heap' 
        ELSE 'Clustered Table' 
    END 
    'Table Type' , 
    C.row_count 
    FROM sys.partitions A 
    LEFT JOIN sys.dm_db_partition_stats C ON A.object_id = C.object_id 
    AND c.index_id IN ( 0 , 1 ) 
    WHERE A.index_id < 2 
    AND A.object_id = object_id ( @TableName ) 

    --Column Details         

    SELECT 
    column_id , 
    name , 
    TYPE_NAME ( user_type_id ) Col_Datatype , 
    max_length , 
    PRECISION , 
    scale , 
    Is_Identity , 
    CASE 
        WHEN is_nullable = 1 THEN 'Yes' 
        ELSE 'No' 
    END 
    Is_Nullable 
    FROM sys.columns WHERE object_id = object_id ( @TableName ) 

    CREATE TABLE #Temp_Index_Details 
    ( 
        [Object_Id] bigint , 
        [Object_Name] varchar ( 100 ) , 
        Index_ID int , 
        Index_Name varchar ( 100 ) , 
        User_Seeks bigint , 
        User_Scans bigint , 
        User_Lookups bigint , 
        Index_Rows bigint , 
        User_Updates bigint , 
        Index_Lock_Attempt_Count bigint , 
        Index_Lock_Promotion_Count bigint , 
        page_lock_wait_count bigint , 
        page_lock_wait_in_ms bigint , 
        [FillFactor] int , 
        Constraint_Type varchar ( 100 ) , 
        Partition_Number bigint 
    ) 

    INSERT INTO #Temp_Index_Details 
    ( [Object_Id] , 
        [Object_Name] , 
        Index_ID , 
        Index_Name , 
        User_Seeks , 
        User_Scans , 
        User_Lookups , 
        Index_Rows , 
        User_Updates , 
        [FillFactor] , 
        Constraint_Type ) 
    SELECT 
    u.object_id , 
    object_name ( u.object_id ) , 
    i.indid , 
    i.name , 
    u.user_seeks , 
    u.user_scans , 
    u.user_lookups , 
    i.rowcnt , 
    u.user_updates , 
    i.OrigFillFactor , 
    k. [type] AS [constraint type] 
    FROM 
    sys.sysindexes i 
    LEFT JOIN sys.dm_db_index_usage_stats u ON u.object_id = i.id 
    AND u.index_id = i.indid 
    LEFT OUTER JOIN sys.key_constraints k ON i.id = k.parent_object_id 
    AND i.indid = k.unique_index_id 
    WHERE u.database_id = db_id ( ) 
    AND object_name ( u.object_id ) = @TableName 
    ORDER BY object_name ( u.object_id ) , 
    i.name , 
    u.user_updates DESC ; 

    UPDATE B SET B.Partition_Number = A.partition_number , 
    B.Index_Lock_Attempt_Count = index_lock_promotion_attempt_count , 
    B.Index_Lock_Promotion_Count = A.index_lock_promotion_count , 
    B.page_lock_wait_count = A.page_lock_wait_count , 
    B.page_lock_wait_in_ms = A.page_lock_wait_in_ms 
    FROM sys.dm_db_index_operational_stats 
    ( db_id ( ) , object_id ( @TableName ) , NULL , NULL ) A 
    INNER JOIN #Temp_Index_Details B ON A.object_id = B.Object_Id AND A.index_id = B.Index_ID 

    --Index Details 
    SELECT 
    A.name IndexName , 
    A.index_id , 
    c.name ColumnName , 
    CASE 
        WHEN System_type_id = User_Type_id THEN Type_Name ( System_type_id ) 
        ELSE Type_Name ( User_Type_id ) 
    END 
    Data_type , 
    A.type_desc , 
    is_identity , 
    is_included_column , 
    is_replicated , 
    Is_Unique , 
    Is_Primary_Key , 
    IS_Unique_Constraint , 
    Fill_factor , 
    is_Padded , 
    is_replicated , 
    is_nullable , 
    is_computed 
    FROM 
    sys.indexes A 
    INNER JOIN sys.index_columns B ON A.object_id = B.object_id 
    AND A.index_id = B.index_id 
    INNER JOIN sys.columns C ON c.object_id = B.object_id 
    AND C.column_id = B.column_id 
    WHERE A.Object_ID = object_id ( @TableName ) 
    ORDER BY A.Index_id , 
    Is_included_Column , 
    Key_ordinal ASC 

    --Index Usage Details         

    SELECT 
    [Object_Name] , 
    Index_ID , 
    Index_Name , 
    User_Seeks , 
    User_Scans , 
    User_Lookups , 
    Index_Rows , 
    User_Updates , 
    Index_Lock_Attempt_Count , 
    Index_Lock_Promotion_Count , 
    page_lock_wait_count , 
    page_lock_wait_in_ms , 
    [FillFactor] , 
    Constraint_Type , 
    Partition_Number 
    FROM #Temp_Index_Details 

    --Stats information 

    SELECT 
    A.object_id , 
    A.name , 
    A.stats_id , 
    C.column_id , 
    C.name , 
    A.auto_created , 
    user_created , 
    has_filter , 
    stats_date ( A.object_id , A.stats_id ) 'Stats_Date' 
    FROM sys.stats A 
    INNER JOIN sys.stats_columns B ON A.object_id = B.object_id 
    AND A.stats_id = B.stats_id 
    INNER JOIN sys.columns C ON B.column_id = C.column_id 
    AND A.object_id = C.object_id 
    WHERE A.object_id = object_id ( @Tablename ) 

    --Primary Key and Foreign Key information 

    SELECT 
    tc.TABLE_NAME AS PrimaryKeyTable , 
    tc.CONSTRAINT_NAME AS PrimaryKey , 
    coalesce ( rc1.CONSTRAINT_NAME , 'N/A' ) AS ForeignKey , 
    coalesce ( tc2.TABLE_NAME , 'N/A' ) AS ForeignKeyTable 
    FROM 
    INFORMATION_SCHEMA.TABLE_CONSTRAINTS tc 
    LEFT JOIN INFORMATION_SCHEMA.REFERENTIAL_CONSTRAINTS rc1 ON tc.CONSTRAINT_NAME = rc1.UNIQUE_CONSTRAINT_NAME 
    LEFT JOIN INFORMATION_SCHEMA.TABLE_CONSTRAINTS tc2 ON tc2.CONSTRAINT_NAME = rc1.CONSTRAINT_NAME 
    WHERE TC.CONSTRAINT_TYPE = 'PRIMARY KEY' 
    AND object_id ( tc.TABLE_NAME ) = object_id ( @Tablename ) 
    ORDER BY tc.TABLE_NAME , 
    tc.CONSTRAINT_NAME , 
    rc1.CONSTRAINT_NAME 

    DROP TABLE #Temp_Index_Details 
END